@@ -1,15 +1,17 @@
-## Get the Constant for Consistency for the M-Scale Using the Bisquare Rho Function
-## @param delta desired breakdown point (between 0 and 0.5)
-##
-## @return consistency constant
+#' Get the Constant for Consistency for the M-Scale Using the Bisquare Rho Function
+#' @param delta desired breakdown point (between 0 and 0.5)
+#'
+#' @return consistency constant
+#' @keywords internal
 #' @importFrom stats pnorm uniroot
+#' @importFrom rlang abort
 .bisquare_consistency_const <- function (delta) {
   ##
   ## Pre-computed values for some delta values
   ##
   eps <- sqrt(.Machine$double.eps)
   if (!isTRUE(delta < 0.5 + eps && delta > -eps)) {
-    stop("`delta` is outside valid bounds")
+    abort("`delta` is outside valid bounds")
   }
 
   if (abs(delta - 0.5) < eps) {
@@ -39,28 +41,36 @@
   uniroot(expectation, interval = integral_interval, delta)$root
 }
 
-## Determine a breakdown point with stable numerical properties of the M-scale
-## with Tukey's bisquare rho function.
-##
-## The M-scale objective (and hence the S-loss) can have unbounded or very high
-## 1st derivative. This can lead to numerical instability of the algorithms and
-## in turn excessive computation time.
-## This function chooses the breakdown point with lowest upper bound of the 1st
-## derivative from a range of bdp's in the vicinity of the desired bdp.
-##
-## @param n number of observations in the sample
-## @param desired_bdp the desired breakdown point (between 0.05 and 0.5)
-## @param tolerance how far can the chosen bdp be away from the desired bdp.
-##                  The chosen bdp is guaranteed to be in the range given by `interval`.
-## @param interval restrict the chosen bdp to this interval.
-## @param precision granularity of the grid of considered bdp's.
+#' Determine a breakdown point with stable numerical properties of the M-scale
+#' with Tukey's bisquare rho function.
+#'
+#' The M-scale objective (and hence the S-loss) can have unbounded or very high
+#' 1st derivative. This can lead to numerical instability of the algorithms and
+#' in turn excessive computation time.
+#' This function chooses the breakdown point with lowest upper bound of the 1st
+#' derivative from a range of bdp's in the vicinity of the desired bdp.
+#'
+#' @param n number of observations in the sample
+#' @param desired_bdp the desired breakdown point (between 0.05 and 0.5)
+#' @param tolerance how far can the chosen bdp be away from the desired bdp.
+#'                  The chosen bdp is guaranteed to be in the range given by `interval`.
+#' @param interval restrict the chosen bdp to this interval.
+#' @param precision granularity of the grid of considered bdp's.
+#' @importFrom rlang warn
+#' @keywords internal
 .find_stable_bdb_bisquare <- function (n, desired_bdp, tolerance = 0.01, precision = 1e-4,
                                        interval = c(0.05, 0.5)) {
+  if (isTRUE(attr(desired_bdp, 'fixed', TRUE))) {
+    return(desired_bdp)
+  }
+
   numeric_tol <- sqrt(.Machine$double.eps)
 
-  bdp_range <- seq(from = max(desired_bdp - tolerance, interval[[1L]]),
-                   to = min(desired_bdp + tolerance, interval[[2L]]),
-                   by = precision)
+  from <- min(max(desired_bdp - tolerance, interval[[1L]]),
+              interval[[2L]])
+  to <- max(min(desired_bdp + tolerance, interval[[2L]]),
+            interval[[1L]])
+  bdp_range <- seq(from, to, by = precision)
 
   # Filter bdp's where the 1st derivative is unbounded
   bdp_range <- bdp_range[abs(bdp_range * n - floor(bdp_range * n)) > numeric_tol]
@@ -80,21 +90,22 @@
   good_bounds <- which(is.finite(first_deriv_bound))
 
   if (length(good_bounds) == 0L) {
-    warning(paste("The chosen breakdown point may lead to numerical instability and",
-                  "excessive computation time.",
-                  "Consider changing the breakdown point via argument `bdp`."))
+    warn(paste("The chosen breakdown point may lead to numerical instability and",
+               "excessive computation time.",
+               "Consider changing the breakdown point via argument `bdp`."))
     return(desired_bdp)
   }
   bdp_range[[which.min(first_deriv_bound)]]
 }
 
-## Approximate Value Matching
-##
-## @param x,table see [base::match] for details.
-## @param eps numerical tolerance for matching.
-## @return a vector the same length as `x` with integers giving the position in
-##         `table` of the first match if there is a match, or `NA_integer_`
-##         otherwise.
+#' Approximate Value Matching
+#'
+#' @param x,table see [base::match] for details.
+#' @param eps numerical tolerance for matching.
+#' @return a vector the same length as `x` with integers giving the position in
+#'         `table` of the first match if there is a match, or `NA_integer_`
+#'         otherwise.
+#' @keywords internal
 .approx_match <- function(x, table, eps) {
   if (missing(eps)) {
     eps <- max(.Machine$double.eps, min(sqrt(.Machine$double.eps), 0.5 * min(x, table)))
@@ -131,19 +142,25 @@ extract_metric <- function (metrics, attr, node) {
   return(estimates)
 }
 
-## Run replicated K-fold CV with random splits
-##
-## @param std_data standardized full data set (standardized by `.standardize_data`)
-## @param cv_k number of folds per CV split
-## @param cv_repl number of CV replications.
-## @param cv_est_fun function taking the standardized training set and the indices of
-##    the left-out observations and returns a list of estimates.
-##    The function always needs to return the same number of estimates!
-## @param metric function taking a vector of prediction errors and returning the scale
-##    of the prediction error.
+#' Run replicated K-fold CV with random splits
+#'
+#' @param std_data standardized full data set
+#'    (standardized by `.standardize_data`)
+#' @param cv_k number of folds per CV split
+#' @param cv_repl number of CV replications.
+#' @param cv_est_fun function taking the standardized training set and
+#'    the indices of the left-out observations and returns a list of estimates.
+#'    The function always needs to return the same number of estimates!
+#' @param metric function taking a vector of prediction errors and
+#'    returning the scale of the prediction error.
+#' @param par_cluster parallel cluster to parallelize computations.
+#' @param handler_args additional arguments to the handler function.
 #' @importFrom Matrix drop
 #' @importFrom rlang abort
-.run_replicated_cv <- function (std_data, cv_k, cv_repl, cv_est_fun, metric, par_cluster = NULL) {
+#' @keywords internal
+.run_replicated_cv <- function (std_data, cv_k, cv_repl, cv_est_fun, metric,
+                                par_cluster = NULL,
+                                handler_args = list()) {
   est_fun <- match.fun(cv_est_fun)
   call_with_errors <- isTRUE(length(formals(metric)) == 1L)
 
@@ -152,25 +169,29 @@ extract_metric <- function (metrics, attr, node) {
   }
 
   test_segments_list <- lapply(integer(cv_repl), function (repl_id) {
-    split(seq_along(std_data$y), sample(rep_len(seq_len(cv_k), length(std_data$y))))
+    split(seq_along(std_data$y),
+          sample(rep_len(seq_len(cv_k), length(std_data$y))))
   })
-  test_segments <- unlist(test_segments_list, recursive = FALSE, use.names = FALSE)
+  test_segments <- unlist(test_segments_list, recursive = FALSE,
+                          use.names = FALSE)
 
   cl_handler <- .make_cluster_handler(par_cluster)
 
-  predictions_all <- cl_handler(test_segments, function (test_ind, est_fun) {
-    train_x <- std_data$x[-test_ind, , drop = FALSE]
-    train_y <- std_data$y[-test_ind]
-    test_x <- std_data$x[test_ind, , drop = FALSE]
+  predictions_all <- cl_handler(
+    test_segments,
+    function (test_ind, est_fun, handler_args) {
+      train_x <- std_data$x[-test_ind, , drop = FALSE]
+      train_y <- std_data$y[-test_ind]
+      test_x <- std_data$x[test_ind, , drop = FALSE]
 
-    train_std <- std_data$cv_standardize(train_x, train_y)
-    cv_ests <- est_fun(train_std, test_ind)
+      train_std <- std_data$cv_standardize(train_x, train_y)
+      cv_ests <- est_fun(train_std, test_ind, handler_args)
 
-    matrix(unlist(lapply(cv_ests, function (est) {
-      unstd_est <- train_std$unstandardize_coef(est)
-      drop(test_x %*% unstd_est$beta) - unstd_est$intercept
-    }), use.names = FALSE, recursive = FALSE), ncol = length(cv_ests))
-  }, est_fun = est_fun)
+      matrix(unlist(lapply(cv_ests, function (est) {
+        unstd_est <- train_std$unstandardize_coef(est)
+        drop(test_x %*% unstd_est$beta) - unstd_est$intercept
+      }), use.names = FALSE, recursive = FALSE), ncol = length(cv_ests))
+    }, est_fun = est_fun, handler_args = handler_args)
 
   predictions_all <- split(predictions_all, rep(seq_len(cv_repl), each = cv_k))
   prediction_metrics <- mapply(
@@ -188,30 +209,34 @@ extract_metric <- function (metrics, attr, node) {
   matrix(unlist(prediction_metrics, recursive = FALSE, use.names = FALSE), ncol = cv_repl)
 }
 
-## Standardize data
-##
-## @param x predictor matrix. Can also be a list with components `x` and `y`,
-##    in which case `y` is ignored.
-## @param y response vector.
-## @param intercept is an intercept included (i.e., should `y` be centered?)
-## @param standardize standardize or not.
-## @param robust use robust standardization.
-## @param location_rho,location_cc rho function and cutoff supplied to `mlocscale()` and `mloc()`
-## @param ... passed on to `mlocscale()`.
-## @return a list with the following entries:
+#' Standardize data
+#'
+#' @param x predictor matrix. Can also be a list with components `x` and `y`,
+#'    in which case `y` is ignored.
+#' @param y response vector.
+#' @param intercept is an intercept included (i.e., should `y` be centered?)
+#' @param standardize standardize or not.
+#' @param robust use robust standardization.
+#' @param location_rho rho function for location estimate
+#' @param cc cutoff value for the rho functions used in scale and location
+#'  estimates.
+#' @param ... passed on to `mlocscale()`.
+#' @return a list with the following entries:
 #' @importFrom Matrix drop
 #' @importFrom methods is
 #' @importFrom rlang abort
 #' @importFrom stats sd
-.standardize_data <- function (x, y, intercept, standardize, robust, sparse, mscale_opts,
-                               location_rho = 'bisquare', location_cc = 4.5,
+#' @keywords internal
+.standardize_data <- function (x, y, intercept, standardize, robust, sparse,
+                               mscale_opts, location_rho = 'bisquare', cc,
                                target_scale_x = NULL, ...) {
   if (is.list(x) && !is.null(x$x) && !is.null(x$y)) {
     y <- x$y
     x <- x$x
   }
 
-  ret_list <- list(scale_x = rep.int(1, ncol(x)), mux = numeric(ncol(x)), muy = 0, x = x, y = y)
+  ret_list <- list(scale_x = rep.int(1, ncol(x)), mux = numeric(ncol(x)),
+                   muy = 0, x = x, y = y)
 
   ## Center data for numerical convenience
   if (isTRUE(intercept)) {
@@ -220,9 +245,16 @@ extract_metric <- function (metrics, attr, node) {
       ret_list$muy <- mean(y)
     } else {
       ret_list$mux <- apply(x, 2, function (xj) {
-        mloc(xj, rho = location_rho, cc = location_cc, opts = mscale_opts)
+        mloc(xj, rho = location_rho, cc = cc, opts = mscale_opts)
       })
-      ret_list$muy <- mloc(y, rho = location_rho, cc = location_cc, opts = mscale_opts)
+      # Center the response using the S-estimate of regression for the
+      # 0-slope.
+      y_locscale <- mlocscale(y, location_rho = location_rho, location_cc = cc,
+                              scale_cc = cc, opts = mscale_opts, ...)
+      if (!isTRUE(y_locscale[['scale']] > .Machine$double.eps)) {
+        abort("M-scale of response is 0.")
+      }
+      ret_list$muy <- y_locscale[['location']]
       if (!is.finite(ret_list$muy)) {
         # In case the response has more than 50% equal values.
         ret_list$muy <- 0
@@ -239,24 +271,27 @@ extract_metric <- function (metrics, attr, node) {
       apply(ret_list$x, 2, sd)
     } else {
       locscale <- apply(ret_list$x, 2, function (xj) {
-        mlocscale(xj, location_rho = location_rho, location_cc = location_cc,
-                  opts = mscale_opts, ...)
+        mlocscale(xj, location_rho = location_rho, location_cc = cc,
+                  scale_cc = cc, opts = mscale_opts, ...)
       })
       if (isTRUE(intercept)) {
         # Re-center the predictors with the updated centers
         ret_list$mux <- ret_list$mux + locscale[1L, ]
-        ret_list$x <- sweep(x, 2L, ret_list$mux, FUN = `-`, check.margin = FALSE)
+        ret_list$x <- sweep(x, 2L, ret_list$mux, FUN = `-`,
+                            check.margin = FALSE)
       }
       locscale[2L, ]
     }
 
     if (!isTRUE(all(ret_list$scale_x > 0))) {
-      abort("Standardization failed. One or more variables in `x` have a scale of 0.")
+      abort(paste("Standardization failed. One or more variables in `x`",
+                  "have a scale of 0."))
     }
 
     if (isTRUE(standardize)) {
       ret_list$x <- if (!is.null(target_scale_x)) {
-        sweep(ret_list$x, 2L, target_scale_x / ret_list$scale_x, FUN = `*`, check.margin = FALSE)
+        sweep(ret_list$x, 2L, target_scale_x / ret_list$scale_x, FUN = `*`,
+              check.margin = FALSE)
       } else {
         sweep(ret_list$x, 2L, ret_list$scale_x, FUN = `/`, check.margin = FALSE)
       }
@@ -278,10 +313,11 @@ extract_metric <- function (metrics, attr, node) {
       # In case of "CV only" standardization, match the original scaling
       .standardize_data(x, y,
                         intercept = intercept,
-                        standardize = TRUE, robust = robust,
+                        standardize = TRUE,
+                        robust = robust,
                         sparse = sparse,
                         location_rho = location_rho,
-                        location_cc = location_cc,
+                        cc = cc,
                         mscale_opts = mscale_opts,
                         target_scale_x = ret_list$scale_x,
                         ... = ...)
@@ -292,7 +328,7 @@ extract_metric <- function (metrics, attr, node) {
                         robust = robust,
                         sparse = sparse,
                         location_rho = location_rho,
-                        location_cc = location_cc,
+                        cc = cc,
                         mscale_opts = mscale_opts,
                         ... = ...)
     }
@@ -307,7 +343,8 @@ extract_metric <- function (metrics, attr, node) {
     }
     if (isTRUE(intercept)) {
       # Adjust intercept
-      coef_obj$intercept <- coef_obj$intercept - ret_list$muy + sum(ret_list$mux * coef_obj$beta)
+      coef_obj$intercept <- coef_obj$intercept - ret_list$muy +
+        sum(ret_list$mux * coef_obj$beta)
     }
     if (isTRUE(standardize)) {
       coef_obj$beta <- coef_obj$beta * (ret_list$scale_x / target_scale_x)
@@ -328,7 +365,8 @@ extract_metric <- function (metrics, attr, node) {
       coef_obj$std_intercept <- coef_obj$intercept
 
       if (isTRUE(standardize)) {
-        coef_obj$beta@x <- coef_obj$beta@x * target_scale_x / ret_list$scale_x[coef_obj$beta@i]
+        coef_obj$beta@x <- coef_obj$beta@x * target_scale_x /
+          ret_list$scale_x[coef_obj$beta@i]
       }
       if (isTRUE(intercept)) {
         # Recreate intercept
@@ -354,7 +392,8 @@ extract_metric <- function (metrics, attr, node) {
       }
       if (isTRUE(intercept)) {
         # Recreate intercept
-        coef_obj$intercept <- coef_obj$intercept + ret_list$muy - sum(ret_list$mux * coef_obj$beta)
+        coef_obj$intercept <- coef_obj$intercept + ret_list$muy -
+          sum(ret_list$mux * coef_obj$beta)
       }
       return(coef_obj)
     }
@@ -592,3 +631,17 @@ extract_metric <- function (metrics, attr, node) {
     list(se_type = se_str, se_fact = se_fact)
   }
 }
+
+## Filter a list to only include items with matching values.
+.filter_list <- function (x, what, value, eps = sqrt(.Machine$double.eps),
+                          comp_fun, ...) {
+  comp_fun <- if (missing(comp_fun)) {
+    function (v) { abs(v - value) < eps }
+  } else {
+    match.fun(comp_fun)
+  }
+  matches <- vapply(x, FUN.VALUE = logical(1L), FUN = function (el) {
+    comp_fun(el[[what]], ...)
+  })
+  x[matches]
+}
